<?php

namespace Drupal\transform_api\Plugin\Transform\Field;

use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;

/**
 * @FieldTransform(
 *  id = "image_style",
 *  label = @Translation("Image Style"),
 *  field_types = {
 *    "image"
 *  }
 * )
 */
class ImageTransform extends \Drupal\transform_api\FieldTransformBase {

  public function transformElements(FieldItemListInterface $items, $langcode): array {
    $values = [];
    /** @var FieldItemInterface $item */
    foreach ($items as $delta => $item) {
      if (!empty($item->getValue()['target_id'])) {
        // File ID.
        $fid = $item->getValue()['target_id'];
        // Load file.
        $file = File::load($fid);
        // Get origin image URI.
        $image_uri = $file->getFileUri();
        // Load image style
        $style = ImageStyle::load($this->getSetting('image_style_id'));
        if (empty($style)) {
          $style = ImageStyle::load('thumbnail');
        }
        // Get URL.
        $url = $style->buildUrl($image_uri);
        $values[] = [
          'src' => $url,
          'alt' => $item->getValue()['alt'] ?? '',
          'title' => $item->getValue()['title'] ?? ''
        ];
      } else {
        $values[] = NULL;
      }
    }
    return $values;
  }

}
