<?php

namespace Drupal\transform_api\Plugin\Transform\Field;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\transform_api\FieldTransformBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @FieldTransform(
 *  id = "text_format",
 *  label = @Translation("Text (formatted)"),
 *  field_types = {
 *    "text_format",
 *    "text_long",
 *    "text_with_summary"
 *  }
 * )
 */
class TextFormatTransform extends FieldTransformBase {

  /**
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $transform_mode, array $third_party_settings, RendererInterface $renderer) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $transform_mode, $third_party_settings);
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['transform_mode'],
      $configuration['third_party_settings'],
      $container->get('renderer')
    );
  }

  public function transformElements(FieldItemListInterface $items, $langcode): array {
    $values = [];
    /** @var FieldItemInterface $item */
    foreach ($items as $delta => $item) {
      if (!empty($item->getValue()['value'])) {
        $build = [
          '#type' => 'processed_text',
          '#text' => $item->value,
          '#format' => $item->format,
          '#langcode' => $item->getLangcode(),
        ];
        $render = $this->renderer->renderPlain($build);
        $values[] = $render;
      }
    }
    return $values;
  }

}
