<?php

namespace Drupal\transform_api\Transform;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\transform_api\Entity\EntityTransformDisplay;
use Drupal\transform_api\FieldTransformInterface;
use Drupal\transform_api\Repository\EntityTransformRepositoryInterface;

class FieldTransform extends TransformBase {

  /**
   * @var \Drupal\transform_api\FieldTransformInterface
   */
  protected $fieldTransform;

  /**
   * @var EntityInterface
   */
  protected $entity;

  protected $field_name;


  public function __construct(FieldTransformInterface $fieldTransform, FieldableEntityInterface $entity, $field_name) {
    $this->entity = $entity;
    $this->fieldTransform = $fieldTransform;
    $this->values = [
      'entity_type' => $entity->getEntityTypeId(),
      'id' => $entity->id(),
      'field' => $field_name,
      'transform_mode' => $fieldTransform->getTransformMode()
    ];
    $this->addCacheableDependency($entity);
    $this->addCacheableDependency(EntityTransformDisplay::collectTransformDisplay($entity, $fieldTransform->getTransformMode()));
  }

  public function getTransformType() {
    return 'field';
  }

  public function transform() {
    return $this->fieldTransform->transform($this->entity->get($this->values['field']), $this->entity->language()->getId());
  }

  public static function createFromEntity(FieldableEntityInterface $entity, $field_name, $transform_mode) {
    /** @var \Drupal\transform_api\FieldTransformManager $fieldTransformManager */
    $fieldTransformManager = \Drupal::service('plugin.manager.transform_api.field_transform');
    $display = EntityTransformDisplay::collectTransformDisplay($entity, $transform_mode);
    $definition = $display->getComponent($field_name);
    $configuration = [
      'field_definition' => $entity->get($field_name)->getFieldDefinition(),
      'settings' => $definition['settings'] ?? [],
      'label' => $definition['label'] ?? '',
      'transform_mode' => $transform_mode ?? EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE,
      'third_party_settings' => $definition['third_party_settings'] ?? []
    ];
    $plugin = $fieldTransformManager->createInstance($definition['type'], $configuration);
    $plugin->prepareTransform([$entity->id() => $entity->get($field_name)]);
    return new self($plugin, $entity, $field_name);
  }

}
